/**
 * @author Olgierd Humeńczuk olgierdh@gmail.com
 * @file main
 */

#include <iostream>
#include <stdlib.h>
#include <stdexcept>
#include <vector>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <SDL.h>
#else
#include <SDL/SDL.h>
#endif

#include <GL/gl.h>

#undef main

/**
 * @brief Funkcja przetwarza zdarzenia pochodzace z kontrolerów takich jak:
 *          * klawiatura
 *          * mysz komputerowa
 *
 * @return true jeżeli aplikacja ma pozostać otwarta, false wpp
 */
bool processInput( void )
{
    // zmienna pomocnicza
    SDL_Event event;

    // czy są jakieś czekające na przetworzenie zdarzenia ?
    if ( SDL_PollEvent( &event ) )
    {
        // sprawdzenie typu zdarzenia
        switch( event.type )
        {
            // wyjście z aplikacji
            case SDL_QUIT:
                return false;

            // użytkownik nacisnął któryś z klawiszy na klawiaturze
            case SDL_KEYDOWN:
                // jeżeli nacisnał klawisz eskape zamykamy aplikacje
                if( event.key.keysym.sym == SDLK_ESCAPE )
                {
                    return false;
                }
                break;
            case SDL_MOUSEMOTION:
                break;
        }
    }
    return true;
}
/**
 * @brief Funkcja odpowiadająca za inicjalizację biblioteki SDL
 *          oraz utworzenie okna z kontekstem OpenGL.
 * @param scrWidth szerokosc ekranu
 * @param scrHeight wysokosc ekranu
 *
 * @throw std::runtime_error jeżeli nastąpi jakikolwiek błąd.
 */
void initSDL( int scrWidth, int scrHeight )
{
    // zaincjalizowanie części biblioteki odpowiadającej za wyświetlanie
    if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
    {
        throw std::runtime_error( "Can't initialize SDL\n" );
    }

    // ustawienie ilości bitów dla każdego z kolorów
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );

    // ustawienie ilości bitów dla specjalnych buforów
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 24 );
    SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 8 );

    // włączenie podwójnego buforowania
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, true );
    SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 1 );

    // stworzenie okna aplikacji z kontekstem OpenGL'a
    if ( SDL_SetVideoMode( scrWidth, scrHeight, 24, SDL_OPENGL ) == 0 )
    {
        throw std::runtime_error(
            std::string( "Unable to set video mode: " ).append(
            	std::string( SDL_GetError() ) ) );
    }
}

/**
 * @brief Główna metoda programu ( entry point )
 * @param argc ilość argumentów przekazanych do programu
 * @param argv tablica wartości argumentów
 */
int main( const int /*argc*/, const char** /*argv*/ )
{
	 const int screenX = 800;
	 const int screenY = 600;

    try
    {
        initSDL( screenX, screenY );

        // dopóki piłka w grze...
        while( processInput() )
        {
            //tutaj obsługa pętli gry
        }
    }
    catch( const std::exception& e )
    {
        std::cerr << "Exception caught: " << e.what() << std::endl;
    }

    SDL_Quit();
    return 0;
}
